#ifdef GL_ES
precision mediump float;
#endif

varying vec2 v_texCoord;
varying vec4 v_fragmentColor;

uniform float u_h_to;
uniform float u_s_to;
uniform float u_v_to;

float min3(float a, float b, float c)
{
    float min = c;
    if(min > a)
    {
        min = a;
    }
    if(min > b)
    {
        min = b;
    }
    return min;
}

float max3(float a, float b, float c)
{
    float max = c;
    if(max < a)
    {
        max = a;
    }
    if(max < b)
    {
        max = b;
    }
    return max;
}

void rgb2hsb(float r, float g, float b, out float h, out float s, out float v)
{
    float rgbMax = max3(r, g, b);
    float rgbMin = min3(r, g, b);
    
    if(rgbMax == rgbMin)
    {
        h = 0.0;
    }
    else
    {
        if(r == rgbMax)
        {
            h = (g-b)/(rgbMax-rgbMin)/6.0;
            if(h < 0.0)
                h = 1.0 + h;
        }
        else if(g == rgbMax)
        {
            h = (b-r)/(rgbMax-rgbMin)/6.0 + 1.0/3.0;
        }
        else
        {
            h = (r-g)/(rgbMax-rgbMin)/6.0 + 2.0/3.0;
        }
    }
    
    if(0.0 == rgbMax)
    {
        s = 0.0;
    }
    else
    {
        s = (rgbMax - rgbMin) / rgbMax;
    }
    
    v = rgbMax;
}
void hsb2rgb(float h, float s, float v, out float r, out float g, out float b)
{
    float hi = floor(h*6.0);
    float f = h * 6.0 - hi;
    float p = v * (1.0 - s);
    float q = v * (1.0 - f * s);
    float t = v * (1.0 - (1.0 - f) * s);
    if(0.0 == hi)
    {
        r = v;
        g = t;
        b = p;
    }
    else if(1.0 == hi)
    {
        r = q;
        g = v;
        b = p;
    }
    else if(2.0 == hi)
    {
        r = p;
        g = v;
        b = t;
    }
    else if(3.0 == hi)
    {
        r = p;
        g = q;
        b = v;
    }
    else if(4.0 == hi)
    {
        r = t;
        g = p;
        b = v;
    }
    else if(5.0 == hi)
    {
        r = v;
        g = p;
        b = q;
    }
    else
    {
        discard;
    }
}
void hsbTo(inout float r, inout float g, inout float b, float h_to, float s_to, float v_to)
{
    float h = 0.0;
    float s = 0.0;
    float v = 0.0;
    
    rgb2hsb(r, g, b, h, s, v);
    
    h = h_to;
    if(h < 0.0)
        h = 0.0;
    else if(h >= 1.0)
        h = 0.0;
        
    s = s_to;
    if(s < 0.0)
        s = 0.0;
    else if(s > 1.0)
        s = 1.0;
    
    v = v_to;
    if(v < 0.0)
        v = 0.0;
    else if(v > 1.0)
        v = 1.0;
    
    hsb2rgb(h, s, v, r, g, b);
}
void main()
{
    vec4 c = texture2D(CC_Texture0, v_texCoord);
    
    hsbTo(c.r, c.g, c.b, u_h_to, u_s_to, u_v_to);
    
    gl_FragColor = c;
}

