#ifdef GL_ES
precision mediump float;
#endif

varying vec2 v_texCoord;
varying vec4 v_fragmentColor;

uniform float u_h_by;
uniform float u_s_by;
uniform float u_l_by;

float min3(float a, float b, float c)
{
    float min = c;
    if(min > a)
    {
        min = a;
    }
    if(min > b)
    {
        min = b;
    }
    return min;
}

float max3(float a, float b, float c)
{
    float max = c;
    if(max < a)
    {
        max = a;
    }
    if(max < b)
    {
        max = b;
    }
    return max;
}

void checkOne(inout float x)
{
    if (x < 0.0)
        x = 0.0;
    else if (x > 1.0)
        x = 1.0;
}

void rgb2hsl(float r, float g, float b, out float h, out float s, out float l)
{
    float rgbMax = max3(r, g, b);
    float rgbMin = min3(r, g, b);
    
    if(rgbMax == rgbMin)
    {
        h = 0.0;
    }
    else
    {
        if(r == rgbMax)
        {
            h = (g-b)/(rgbMax-rgbMin)/6.0;
            if(h < 0.0)
                h = 1.0 + h;
        }
        else if(g == rgbMax)
        {
            h = (b-r)/(rgbMax-rgbMin)/6.0 + 1.0/3.0;
        }
        else
        {
            h = (r-g)/(rgbMax-rgbMin)/6.0 + 2.0/3.0;
        }
    }
    
    l = (rgbMax + rgbMin) / 2.0;
    
    if(0.0 == l || rgbMax == rgbMin)
    {
        s = 0.0;
    }
    else
    {
        if(l < 0.5)
            s = (rgbMax - rgbMin) / (rgbMax + rgbMin);
        else
            s = (rgbMax - rgbMin) / (2.0 - rgbMax - rgbMin);
    }
}
float qpthsl2rgb(float h, float s, float l, float q, float p, float t)
{
    if(t < 0.0)
        t = t + 1.0;
    else if(t > 1.0)
        t = t - 1.0;
    float color = 0.0;
    if(t < 1.0 / 6.0)
    {
        color = p + ((q - p) * 6.0 * t);
    }
    else if(t < 0.5)
    {
        color = q;
    }
    else if(t < 2.0 / 3.0)
    {
        color = p + ((q - p) * 6.0 * (2.0 / 3.0 - t));
    }
    else
    {
        color = p;
    }
    return color;
}
void hsl2rgb(float h, float s, float l, out float r, out float g, out float b)
{
    if(0.0 == s)
    {
        r = l;
        g = l;
        b = l;
    }
    else
    {
        float q = 0.0;
        float p = 0.0;
        if(l < 0.5)
        {
            q = l * (1.0 + s);
        }
        else
        {
            q = l + s - (l * s);
        }
        p = 2.0 * l - q;
        
        r = qpthsl2rgb(h, s, l, q, p, h + 1.0/3.0);
        g = qpthsl2rgb(h, s, l, q, p, h );
        b = qpthsl2rgb(h, s, l, q, p, h - 1.0/3.0);
    }
}
void hslBy(inout float r, inout float g, inout float b, float h_by, float s_by, float v_by)
{
    float h = 0.0;
    float s = 0.0;
    float v = 0.0;
    
    rgb2hsl(r, g, b, h, s, v);
    
    h = h + h_by;
    checkOne(h);
        
    s = s + s_by;
    checkOne(s);
    
    v = v + v_by;
    checkOne(v);
    
    hsl2rgb(h, s, v, r, g, b);
    
    checkOne(r);
    checkOne(g);
    checkOne(b);
}
void main()
{
    vec4 c = texture2D(CC_Texture0, v_texCoord);
    
    hslBy(c.r, c.g, c.b, u_h_by, u_s_by, u_l_by);
    
    gl_FragColor = c;
}

