#ifdef GL_ES
precision mediump float;
#endif

varying vec2 v_texCoord;
varying vec4 v_fragmentColor;

uniform float u_h_to;
uniform float u_s_to;
uniform float u_l_to;

float min3(float a, float b, float c)
{
    float min = c;
    if(min > a)
    {
        min = a;
    }
    if(min > b)
    {
        min = b;
    }
    return min;
}

float max3(float a, float b, float c)
{
    float max = c;
    if(max < a)
    {
        max = a;
    }
    if(max < b)
    {
        max = b;
    }
    return max;
}

void rgb2hsl(float r, float g, float b, out float h, out float s, out float l)
{
    float rgbMax = max3(r, g, b);
    float rgbMin = min3(r, g, b);
    
    if(rgbMax == rgbMin)
    {
        h = 0.0;
    }
    else
    {
        if(r == rgbMax)
        {
            h = (g-b)/(rgbMax-rgbMin)/6.0;
            if(h < 0.0)
                h = 1.0 + h;
        }
        else if(g == rgbMax)
        {
            h = (b-r)/(rgbMax-rgbMin)/6.0 + 1.0/3.0;
        }
        else
        {
            h = (r-g)/(rgbMax-rgbMin)/6.0 + 2.0/3.0;
        }
    }
    
    l = (rgbMax + rgbMin) / 2.0;
    
    if(0.0 == l || rgbMax == rgbMin)
    {
        s = 0.0;
    }
    else
    {
        if(l < 0.5)
            s = (rgbMax - rgbMin) / (rgbMax + rgbMin);
        else
            s = (rgbMax - rgbMin) / (2.0 - rgbMax - rgbMin);
    }
}
float qpthsl2rgb(float h, float s, float l, float q, float p, float t)
{
    if(t < 0.0)
        t = t + 1.0;
    else if(t > 1.0)
        t = t - 1.0;
    
    if(t < 1.0 / 6.0)
    {
        return p + ((q - p) * 6.0 * t);
    }
    else if(t < 0.5)
    {
        return q;
    }
    else if(t < 2.0 / 3.0)
    {
        return p + ((q - p) * 6.0 * (2.0 / 3.0 - t));
    }
    else
    {
        return p;
    }
}
void hsl2rgb(float h, float s, float l, out float r, out float g, out float b)
{
    if(0.0 == s)
    {
        r = l;
        g = l;
        b = l;
    }
    else
    {
        float q = 0.0;
        float p = 0.0;
        if(l < 0.5)
        {
            q = l * (1.0 + s);
        }
        else
        {
            q = l + s - (l * s);
        }
        p = 2.0 * l - q;
        
        r = qpthsl2rgb(h, s, l, q, p, h + 1.0/3.0);
        g = qpthsl2rgb(h, s, l, q, p, h );
        b = qpthsl2rgb(h, s, l, q, p, h - 1.0/3.0);
    }
}
void hslTo(inout float r, inout float g, inout float b, float h_to, float s_to, float l_to)
{
    float h = 0.0;
    float s = 0.0;
    float l = 0.0;
    
    rgb2hsl(r, g, b, h, s, l);
    
    if(0.0 <= h_to)
    {
        h = h_to;
        if(h < 0.0)
            h = 0.0;
        else if(h >= 1.0)
            h = 0.0;
    }

    if(0.0 <= s_to)
    {
        s = s_to;
        if(s < 0.0)
            s = 0.0;
        else if(s > 1.0)
            s = 1.0;
    }

    if(0.0 <= l_to)
    {
        l = l_to;
        if(l < 0.0)
            l = 0.0;
        else if(l > 1.0)
            l = 1.0;
    }
    
    hsl2rgb(h, s, l, r, g, b);
}
void main()
{
    vec4 c = texture2D(CC_Texture0, v_texCoord);
    
    hslTo(c.r, c.g, c.b, u_h_to, u_s_to, u_l_to);
    
    gl_FragColor = c;
}

